@@ -1,14 +1,14 @@
 // Test Sftp filesystem interface
 
-// +build !plan9,go1.9
+// +build !plan9
 
 package sftp_test
 
 import (
 	"testing"
 
-	"github.com/ncw/rclone/backend/sftp"
-	"github.com/ncw/rclone/fstest/fstests"
+	"github.com/rclone/rclone/backend/sftp"
+	"github.com/rclone/rclone/fstest/fstests"
 )
 
 // TestIntegration runs integration tests against the remote
