@@ -1,18 +1,21 @@
 package dbhashsum
 
 import (
+	"context"
 	"os"
 
-	"github.com/ncw/rclone/cmd"
-	"github.com/ncw/rclone/fs/operations"
+	"github.com/rclone/rclone/backend/dropbox/dbhash"
+	"github.com/rclone/rclone/cmd"
+	"github.com/rclone/rclone/fs/hash"
+	"github.com/rclone/rclone/fs/operations"
 	"github.com/spf13/cobra"
 )
 
 func init() {
-	cmd.Root.AddCommand(commandDefintion)
+	cmd.Root.AddCommand(commandDefinition)
 }
 
-var commandDefintion = &cobra.Command{
+var commandDefinition = &cobra.Command{
 	Use:   "dbhashsum remote:path",
 	Short: `Produces a Dropbox hash file for all the objects in the path.`,
 	Long: `
@@ -25,7 +28,8 @@ The output is in the same format as md5sum and sha1sum.
 		cmd.CheckArgs(1, 1, command, args)
 		fsrc := cmd.NewFsSrc(args)
 		cmd.Run(false, false, command, func() error {
-			return operations.DropboxHashSum(fsrc, os.Stdout)
+			dbHashType := hash.RegisterHash("Dropbox", 64, dbhash.New)
+			return operations.HashLister(context.Background(), dbHashType, fsrc, os.Stdout)
 		})
 	},
 }
