@@ -1,13 +1,14 @@
 package hashsum
 
 import (
+	"context"
 	"errors"
 	"fmt"
 	"os"
 
-	"github.com/ncw/rclone/cmd"
-	"github.com/ncw/rclone/fs/hash"
-	"github.com/ncw/rclone/fs/operations"
+	"github.com/rclone/rclone/cmd"
+	"github.com/rclone/rclone/fs/hash"
+	"github.com/rclone/rclone/fs/operations"
 	"github.com/spf13/cobra"
 )
 
@@ -40,7 +41,7 @@ Then
 		cmd.CheckArgs(0, 2, command, args)
 		if len(args) == 0 {
 			fmt.Printf("Supported hashes are:\n")
-			for _, ht := range hash.Supported.Array() {
+			for _, ht := range hash.Supported().Array() {
 				fmt.Printf("  * %v\n", ht)
 			}
 			return nil
@@ -54,7 +55,7 @@ Then
 		}
 		fsrc := cmd.NewFsSrc(args[1:])
 		cmd.Run(false, false, command, func() error {
-			return operations.HashLister(ht, fsrc, os.Stdout)
+			return operations.HashLister(context.Background(), ht, fsrc, os.Stdout)
 		})
 		return nil
 	},
