@@ -17,7 +17,7 @@ import (
 
 var (
 	// Flags
-	iterations   = flag.Int("n", 1E6, "Iterations to try")
+	iterations   = flag.Int("n", 1e6, "Iterations to try")
 	maxBlockSize = flag.Int("b", 1024*1024, "Max block size to read")
 	simultaneous = flag.Int("transfers", 16, "Number of simultaneous files to open")
 	seeksPerFile = flag.Int("seeks", 8, "Seeks per file")
