@@ -3,6 +3,7 @@
 package mounttest
 
 import (
+	"context"
 	"flag"
 	"fmt"
 	"io/ioutil"
@@ -17,11 +18,11 @@ import (
 	"testing"
 	"time"
 
-	_ "github.com/ncw/rclone/backend/all" // import all the backends
-	"github.com/ncw/rclone/fs"
-	"github.com/ncw/rclone/fs/walk"
-	"github.com/ncw/rclone/fstest"
-	"github.com/ncw/rclone/vfs"
+	_ "github.com/rclone/rclone/backend/all" // import all the backends
+	"github.com/rclone/rclone/fs"
+	"github.com/rclone/rclone/fs/walk"
+	"github.com/rclone/rclone/fstest"
+	"github.com/rclone/rclone/vfs"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
 )
@@ -76,6 +77,7 @@ func RunTests(t *testing.T, fn MountFn) {
 			t.Run("TestWriteFileOverwrite", TestWriteFileOverwrite)
 			t.Run("TestWriteFileDoubleClose", TestWriteFileDoubleClose)
 			t.Run("TestWriteFileFsync", TestWriteFileFsync)
+			t.Run("TestWriteFileDup", TestWriteFileDup)
 		})
 		log.Printf("Finished test run with cache mode %v (ok=%v)", cacheMode, ok)
 		if !ok {
@@ -114,12 +116,12 @@ func newRun() *Run {
 	fstest.Initialise()
 
 	var err error
-	r.fremote, r.fremoteName, r.cleanRemote, err = fstest.RandomRemote(*fstest.RemoteName, *fstest.SubDir)
+	r.fremote, r.fremoteName, r.cleanRemote, err = fstest.RandomRemote()
 	if err != nil {
 		log.Fatalf("Failed to open remote %q: %v", *fstest.RemoteName, err)
 	}
 
-	err = r.fremote.Mkdir("")
+	err = r.fremote.Mkdir(context.Background(), "")
 	if err != nil {
 		log.Fatalf("Failed to open mkdir %q: %v", *fstest.RemoteName, err)
 	}
@@ -211,7 +213,7 @@ func (r *Run) cacheMode(cacheMode vfs.CacheMode) {
 	r.vfs.WaitForWriters(30 * time.Second)
 	// Empty and remake the remote
 	r.cleanRemote()
-	err := r.fremote.Mkdir("")
+	err := r.fremote.Mkdir(context.Background(), "")
 	if err != nil {
 		log.Fatalf("Failed to open mkdir %q: %v", *fstest.RemoteName, err)
 	}
@@ -296,7 +298,7 @@ func (r *Run) readLocal(t *testing.T, dir dirMap, filePath string) {
 
 // reads the remote tree into dir
 func (r *Run) readRemote(t *testing.T, dir dirMap, filepath string) {
-	objs, dirs, err := walk.GetAll(r.fremote, filepath, true, 1)
+	objs, dirs, err := walk.GetAll(context.Background(), r.fremote, filepath, true, 1)
 	if err == fs.ErrorDirNotFound {
 		return
 	}
