@@ -3,23 +3,23 @@ package obscure
 import (
 	"fmt"
 
-	"github.com/ncw/rclone/cmd"
-	"github.com/ncw/rclone/fs"
+	"github.com/rclone/rclone/cmd"
+	"github.com/rclone/rclone/fs/config/obscure"
 	"github.com/spf13/cobra"
 )
 
 func init() {
-	cmd.Root.AddCommand(commandDefintion)
+	cmd.Root.AddCommand(commandDefinition)
 }
 
-var commandDefintion = &cobra.Command{
+var commandDefinition = &cobra.Command{
 	Use:   "obscure password",
 	Short: `Obscure password for use in the rclone.conf`,
 	Run: func(command *cobra.Command, args []string) {
 		cmd.CheckArgs(1, 1, command, args)
 		cmd.Run(false, false, command, func() error {
-			obscure := fs.MustObscure(args[0])
-			fmt.Println(obscure)
+			obscured := obscure.MustObscure(args[0])
+			fmt.Println(obscured)
 			return nil
 		})
 	},
