@@ -2,7 +2,7 @@
 
 # error codes
 # 0 - exited without problems
-# 1 - parameters not supported were used or some unexpected error occured
+# 1 - parameters not supported were used or some unexpected error occurred
 # 2 - OS not supported by this script
 # 3 - installed version of rclone is up to date
 # 4 - supported unzip tools are not available
@@ -10,7 +10,7 @@
 set -e
 
 #when adding a tool to the list make sure to also add it's corresponding command further in the script
-unzip_tools_list=('unzip' '7z', 'busybox')
+unzip_tools_list=('unzip' '7z' 'busybox')
 
 usage() { echo "Usage: curl https://rclone.org/install.sh | sudo bash [-s beta]" 1>&2; exit 1; }
 
@@ -25,7 +25,7 @@ fi
 
 
 #create tmp directory and move to it with macOS compatibility fallback
-tmp_dir=`mktemp -d 2>/dev/null || mktemp -d -t 'rclone-install'`; cd $tmp_dir
+tmp_dir=`mktemp -d 2>/dev/null || mktemp -d -t 'rclone-install.XXXXXXXXXX'`; cd $tmp_dir
 
 
 #make sure unzip tool is available and choose one to work with
@@ -142,7 +142,7 @@ esac
 cd $unzip_dir/*
 
 
-#mounting rclone to enviroment
+#mounting rclone to environment
 
 case $OS in
   'linux')
@@ -152,9 +152,13 @@ case $OS in
     chown root:root /usr/bin/rclone.new
     mv /usr/bin/rclone.new /usr/bin/rclone
     #manuals
-    mkdir -p /usr/local/share/man/man1
-    cp rclone.1 /usr/local/share/man/man1/
-    mandb
+    if ! [ -x "$(command -v mandb)" ]; then
+        echo 'mandb not found. The rclone man docs will not be installed.'
+    else 
+        mkdir -p /usr/local/share/man/man1
+        cp rclone.1 /usr/local/share/man/man1/
+        mandb
+    fi
     ;;
   'freebsd'|'openbsd'|'netbsd')
     #bin
