@@ -3,6 +3,7 @@ package filter
 
 import (
 	"bufio"
+	"context"
 	"fmt"
 	"log"
 	"os"
@@ -11,8 +12,9 @@ import (
 	"strings"
 	"time"
 
-	"github.com/ncw/rclone/fs"
 	"github.com/pkg/errors"
+	"github.com/rclone/rclone/fs"
+	"golang.org/x/sync/errgroup"
 )
 
 // Active is the globally active filter
@@ -75,7 +77,7 @@ func (rs *rules) len() int {
 // FilesMap describes the map of files to transfer
 type FilesMap map[string]struct{}
 
-// Opt configues the filter
+// Opt configures the filter
 type Opt struct {
 	DeleteExcluded bool
 	FilterRule     []string
@@ -189,7 +191,12 @@ func NewFilter(opt *Opt) (f *Filter, err error) {
 			return nil, err
 		}
 	}
+
+	inActive := f.InActive()
 	for _, rule := range f.Opt.FilesFrom {
+		if !inActive {
+			return nil, fmt.Errorf("The usage of --files-from overrides all other filters, it should be used alone")
+		}
 		f.initAddFile() // init to show --files-from set even if no files within
 		err := forEachLine(rule, func(line string) error {
 			return f.AddFile(line)
@@ -371,12 +378,12 @@ func (f *Filter) ListContainsExcludeFile(entries fs.DirEntries) bool {
 
 // IncludeDirectory returns a function which checks whether this
 // directory should be included in the sync or not.
-func (f *Filter) IncludeDirectory(fs fs.Fs) func(string) (bool, error) {
+func (f *Filter) IncludeDirectory(ctx context.Context, fs fs.Fs) func(string) (bool, error) {
 	return func(remote string) (bool, error) {
 		remote = strings.Trim(remote, "/")
 		// first check if we need to remove directory based on
 		// the exclude file
-		excl, err := f.DirContainsExcludeFile(fs, remote)
+		excl, err := f.DirContainsExcludeFile(ctx, fs, remote)
 		if err != nil {
 			return false, err
 		}
@@ -403,9 +410,9 @@ func (f *Filter) IncludeDirectory(fs fs.Fs) func(string) (bool, error) {
 // DirContainsExcludeFile checks if exclude file is present in a
 // directroy. If fs is nil, it works properly if ExcludeFile is an
 // empty string (for testing).
-func (f *Filter) DirContainsExcludeFile(fremote fs.Fs, remote string) (bool, error) {
+func (f *Filter) DirContainsExcludeFile(ctx context.Context, fremote fs.Fs, remote string) (bool, error) {
 	if len(f.Opt.ExcludeFile) > 0 {
-		exists, err := fs.FileExists(fremote, path.Join(remote, f.Opt.ExcludeFile))
+		exists, err := fs.FileExists(ctx, fremote, path.Join(remote, f.Opt.ExcludeFile))
 		if err != nil {
 			return false, err
 		}
@@ -442,11 +449,11 @@ func (f *Filter) Include(remote string, size int64, modTime time.Time) bool {
 // IncludeObject returns whether this object should be included into
 // the sync or not. This is a convenience function to avoid calling
 // o.ModTime(), which is an expensive operation.
-func (f *Filter) IncludeObject(o fs.Object) bool {
+func (f *Filter) IncludeObject(ctx context.Context, o fs.Object) bool {
 	var modTime time.Time
 
 	if !f.ModTimeFrom.IsZero() || !f.ModTimeTo.IsZero() {
-		modTime = o.ModTime()
+		modTime = o.ModTime(ctx)
 	} else {
 		modTime = time.Unix(0, 0)
 	}
@@ -506,22 +513,54 @@ func (f *Filter) HaveFilesFrom() bool {
 var errFilesFromNotSet = errors.New("--files-from not set so can't use Filter.ListR")
 
 // MakeListR makes function to return all the files set using --files-from
-func (f *Filter) MakeListR(NewObject func(remote string) (fs.Object, error)) fs.ListRFn {
-	return func(dir string, callback fs.ListRCallback) error {
+func (f *Filter) MakeListR(ctx context.Context, NewObject func(ctx context.Context, remote string) (fs.Object, error)) fs.ListRFn {
+	return func(ctx context.Context, dir string, callback fs.ListRCallback) error {
 		if !f.HaveFilesFrom() {
 			return errFilesFromNotSet
 		}
-		var entries fs.DirEntries
-		for remote := range f.files {
-			entry, err := NewObject(remote)
-			if err == fs.ErrorObjectNotFound {
-				// Skip files that are not found
-			} else if err != nil {
-				return err
-			} else {
-				entries = append(entries, entry)
-			}
+		var (
+			remotes = make(chan string, fs.Config.Checkers)
+			g       errgroup.Group
+		)
+		for i := 0; i < fs.Config.Checkers; i++ {
+			g.Go(func() (err error) {
+				var entries = make(fs.DirEntries, 1)
+				for remote := range remotes {
+					entries[0], err = NewObject(ctx, remote)
+					if err == fs.ErrorObjectNotFound {
+						// Skip files that are not found
+					} else if err != nil {
+						return err
+					} else {
+						err = callback(entries)
+						if err != nil {
+							return err
+						}
+					}
+				}
+				return nil
+			})
 		}
-		return callback(entries)
+		for remote := range f.files {
+			remotes <- remote
+		}
+		close(remotes)
+		return g.Wait()
 	}
 }
+
+// UsesDirectoryFilters returns true if the filter uses directory
+// filters and false if it doesn't.
+//
+// This is used in deciding whether to walk directories or use ListR
+func (f *Filter) UsesDirectoryFilters() bool {
+	if len(f.dirRules.rules) == 0 {
+		return false
+	}
+	rule := f.dirRules.rules[0]
+	re := rule.Regexp.String()
+	if rule.Include == true && re == "^.*$" {
+		return false
+	}
+	return true
+}
