@@ -8,7 +8,7 @@ import (
 	"log"
 	"os"
 
-	"github.com/ncw/rclone/fs/config"
+	"github.com/rclone/rclone/fs/config"
 	"golang.org/x/sys/unix"
 )
 
