@@ -8,22 +8,28 @@ import (
 	"strings"
 	"sync"
 
-	"github.com/ncw/rclone/fs"
-	"github.com/ncw/rclone/fs/filter"
-	"github.com/ncw/rclone/fs/list"
-	"github.com/ncw/rclone/fs/walk"
+	"github.com/pkg/errors"
+
+	"github.com/rclone/rclone/fs"
+	"github.com/rclone/rclone/fs/dirtree"
+	"github.com/rclone/rclone/fs/filter"
+	"github.com/rclone/rclone/fs/list"
+	"github.com/rclone/rclone/fs/walk"
 	"golang.org/x/text/unicode/norm"
 )
 
 // March holds the data used to traverse two Fs simultaneously,
-// calling callback for each match
+// calling Callback for each match
 type March struct {
 	// parameters
-	ctx      context.Context
-	fdst     fs.Fs
-	fsrc     fs.Fs
-	dir      string
-	callback Marcher
+	Ctx           context.Context // context for background goroutines
+	Fdst          fs.Fs           // source Fs
+	Fsrc          fs.Fs           // dest Fs
+	Dir           string          // directory
+	NoTraverse    bool            // don't traverse the destination
+	SrcIncludeAll bool            // don't include all files in the src
+	DstIncludeAll bool            // don't include all files in the destination
+	Callback      Marcher         // object to call with results
 	// internal state
 	srcListDir listDirFn // function to call to list a directory in the src
 	dstListDir listDirFn // function to call to list a directory in the dst
@@ -37,20 +43,15 @@ type Marcher interface {
 	// DstOnly is called for a DirEntry found only in the destination
 	DstOnly(dst fs.DirEntry) (recurse bool)
 	// Match is called for a DirEntry found both in the source and destination
-	Match(dst, src fs.DirEntry) (recurse bool)
+	Match(ctx context.Context, dst, src fs.DirEntry) (recurse bool)
 }
 
-// New sets up a march over fsrc, and fdst calling back callback for each match
-func New(ctx context.Context, fdst, fsrc fs.Fs, dir string, callback Marcher) *March {
-	m := &March{
-		ctx:      ctx,
-		fdst:     fdst,
-		fsrc:     fsrc,
-		dir:      dir,
-		callback: callback,
+// init sets up a march over opt.Fsrc, and opt.Fdst calling back callback for each match
+func (m *March) init() {
+	m.srcListDir = m.makeListDir(m.Fsrc, m.SrcIncludeAll)
+	if !m.NoTraverse {
+		m.dstListDir = m.makeListDir(m.Fdst, m.DstIncludeAll)
 	}
-	m.srcListDir = m.makeListDir(fsrc, false)
-	m.dstListDir = m.makeListDir(fdst, filter.Active.Opt.DeleteExcluded)
 	// Now create the matching transform
 	// ..normalise the UTF8 first
 	m.transforms = append(m.transforms, norm.NFC.String)
@@ -60,33 +61,37 @@ func New(ctx context.Context, fdst, fsrc fs.Fs, dir string, callback Marcher) *M
 	//                  | Yes | No  | No                 |
 	//                  | No  | Yes | Yes                |
 	//                  | Yes | Yes | Yes                |
-	if fdst.Features().CaseInsensitive {
+	if m.Fdst.Features().CaseInsensitive || fs.Config.IgnoreCaseSync {
 		m.transforms = append(m.transforms, strings.ToLower)
 	}
-	return m
 }
 
 // list a directory into entries, err
 type listDirFn func(dir string) (entries fs.DirEntries, err error)
 
-// makeListDir makes a listing function for the given fs and includeAll flags
+// makeListDir makes constructs a listing function for the given fs
+// and includeAll flags for marching through the file system.
 func (m *March) makeListDir(f fs.Fs, includeAll bool) listDirFn {
-	if (!fs.Config.UseListR || f.Features().ListR == nil) && !filter.Active.HaveFilesFrom() {
+	if !(fs.Config.UseListR && f.Features().ListR != nil) && // !--fast-list active and
+		!(fs.Config.NoTraverse && filter.Active.HaveFilesFrom()) { // !(--files-from and --no-traverse)
 		return func(dir string) (entries fs.DirEntries, err error) {
-			return list.DirSorted(f, includeAll, dir)
+			return list.DirSorted(m.Ctx, f, includeAll, dir)
 		}
 	}
+
+	// This returns a closure for use when --fast-list is active or for when
+	// --files-from and --no-traverse is set
 	var (
 		mu      sync.Mutex
 		started bool
-		dirs    walk.DirTree
+		dirs    dirtree.DirTree
 		dirsErr error
 	)
 	return func(dir string) (entries fs.DirEntries, err error) {
 		mu.Lock()
 		defer mu.Unlock()
 		if !started {
-			dirs, dirsErr = walk.NewDirTree(f, m.dir, includeAll, fs.Config.MaxDepth)
+			dirs, dirsErr = walk.NewDirTree(m.Ctx, f, m.Dir, includeAll, fs.Config.MaxDepth)
 			started = true
 		}
 		if dirsErr != nil {
@@ -113,7 +118,9 @@ type listDirJob struct {
 }
 
 // Run starts the matching process off
-func (m *March) Run() {
+func (m *March) Run() error {
+	m.init()
+
 	srcDepth := fs.Config.MaxDepth
 	if srcDepth < 0 {
 		srcDepth = fs.MaxLevel
@@ -123,6 +130,10 @@ func (m *March) Run() {
 		dstDepth = fs.MaxLevel
 	}
 
+	var mu sync.Mutex // Protects vars below
+	var jobError error
+	var errCount int
+
 	// Start some directory listing go routines
 	var wg sync.WaitGroup         // sync closing of go routines
 	var traversing sync.WaitGroup // running directory traversals
@@ -133,13 +144,22 @@ func (m *March) Run() {
 			defer wg.Done()
 			for {
 				select {
-				case <-m.ctx.Done():
+				case <-m.Ctx.Done():
 					return
 				case job, ok := <-in:
 					if !ok {
 						return
 					}
-					jobs := m.processJob(job)
+					jobs, err := m.processJob(job)
+					if err != nil {
+						mu.Lock()
+						// Keep reference only to the first encountered error
+						if jobError == nil {
+							jobError = err
+						}
+						errCount++
+						mu.Unlock()
+					}
 					if len(jobs) > 0 {
 						traversing.Add(len(jobs))
 						go func() {
@@ -147,7 +167,7 @@ func (m *March) Run() {
 							// jobs off for traversal in the background
 							for _, newJob := range jobs {
 								select {
-								case <-m.ctx.Done():
+								case <-m.Ctx.Done():
 									// discard job if finishing
 									traversing.Done()
 								case in <- newJob:
@@ -164,14 +184,14 @@ func (m *March) Run() {
 	// Start the process
 	traversing.Add(1)
 	in <- listDirJob{
-		srcRemote: m.dir,
+		srcRemote: m.Dir,
 		srcDepth:  srcDepth - 1,
-		dstRemote: m.dir,
+		dstRemote: m.Dir,
 		dstDepth:  dstDepth - 1,
 	}
 	go func() {
 		// when the context is cancelled discard the remaining jobs
-		<-m.ctx.Done()
+		<-m.Ctx.Done()
 		for range in {
 			traversing.Done()
 		}
@@ -179,12 +199,17 @@ func (m *March) Run() {
 	traversing.Wait()
 	close(in)
 	wg.Wait()
+
+	if errCount > 1 {
+		return errors.Wrapf(jobError, "march failed with %d error(s): first error", errCount)
+	}
+	return jobError
 }
 
 // Check to see if the context has been cancelled
 func (m *March) aborting() bool {
 	select {
-	case <-m.ctx.Done():
+	case <-m.Ctx.Done():
 		return true
 	default:
 	}
@@ -214,7 +239,7 @@ func (es matchEntries) Less(i, j int) bool {
 	ei, ej := &es[i], &es[j]
 	if ei.name == ej.name {
 		if ei.leaf == ej.leaf {
-			return ei.entry.Remote() < ej.entry.Remote()
+			return fs.CompareDirEntries(ei.entry, ej.entry) < 0
 		}
 		return ei.leaf < ej.leaf
 	}
@@ -268,6 +293,7 @@ type matchTransformFn func(name string) string
 func matchListings(srcListEntries, dstListEntries fs.DirEntries, transforms []matchTransformFn) (srcOnly fs.DirEntries, dstOnly fs.DirEntries, matches []matchPair) {
 	srcList := newMatchEntries(srcListEntries, transforms)
 	dstList := newMatchEntries(dstListEntries, transforms)
+
 	for iSrc, iDst := 0, 0; ; iSrc, iDst = iSrc+1, iDst+1 {
 		var src, dst fs.DirEntry
 		var srcName, dstName string
@@ -283,34 +309,40 @@ func matchListings(srcListEntries, dstListEntries fs.DirEntries, transforms []ma
 			break
 		}
 		if src != nil && iSrc > 0 {
-			prev := srcList[iSrc-1].name
-			if srcName == prev {
+			prev := srcList[iSrc-1].entry
+			prevName := srcList[iSrc-1].name
+			if srcName == prevName && fs.DirEntryType(prev) == fs.DirEntryType(src) {
 				fs.Logf(src, "Duplicate %s found in source - ignoring", fs.DirEntryType(src))
 				iDst-- // ignore the src and retry the dst
 				continue
-			} else if srcName < prev {
+			} else if srcName < prevName {
 				// this should never happen since we sort the listings
 				panic("Out of order listing in source")
 			}
 		}
 		if dst != nil && iDst > 0 {
-			prev := dstList[iDst-1].name
-			if dstName == prev {
+			prev := dstList[iDst-1].entry
+			prevName := dstList[iDst-1].name
+			if dstName == prevName && fs.DirEntryType(dst) == fs.DirEntryType(prev) {
 				fs.Logf(dst, "Duplicate %s found in destination - ignoring", fs.DirEntryType(dst))
 				iSrc-- // ignore the dst and retry the src
 				continue
-			} else if dstName < prev {
+			} else if dstName < prevName {
 				// this should never happen since we sort the listings
 				panic("Out of order listing in destination")
 			}
 		}
 		if src != nil && dst != nil {
-			if srcName < dstName {
-				dst = nil
-				iDst-- // retry the dst
-			} else if srcName > dstName {
+			// we can't use CompareDirEntries because srcName, dstName could
+			// be different then src.Remote() or dst.Remote()
+			srcType := fs.DirEntryType(src)
+			dstType := fs.DirEntryType(dst)
+			if srcName > dstName || (srcName == dstName && srcType > dstType) {
 				src = nil
-				iSrc-- // retry the src
+				iSrc--
+			} else if srcName < dstName || (srcName == dstName && srcType < dstType) {
+				dst = nil
+				iDst--
 			}
 		}
 		// Debugf(nil, "src = %v, dst = %v", src, dst)
@@ -333,8 +365,9 @@ func matchListings(srcListEntries, dstListEntries fs.DirEntries, transforms []ma
 // more jobs
 //
 // returns errors using processError
-func (m *March) processJob(job listDirJob) (jobs []listDirJob) {
+func (m *March) processJob(job listDirJob) ([]listDirJob, error) {
 	var (
+		jobs                   []listDirJob
 		srcList, dstList       fs.DirEntries
 		srcListErr, dstListErr error
 		wg                     sync.WaitGroup
@@ -348,7 +381,7 @@ func (m *March) processJob(job listDirJob) (jobs []listDirJob) {
 			srcList, srcListErr = m.srcListDir(job.srcRemote)
 		}()
 	}
-	if !job.noDst {
+	if !m.NoTraverse && !job.noDst {
 		wg.Add(1)
 		go func() {
 			defer wg.Done()
@@ -361,26 +394,41 @@ func (m *March) processJob(job listDirJob) (jobs []listDirJob) {
 	if srcListErr != nil {
 		fs.Errorf(job.srcRemote, "error reading source directory: %v", srcListErr)
 		fs.CountError(srcListErr)
-		return nil
+		return nil, srcListErr
 	}
 	if dstListErr == fs.ErrorDirNotFound {
 		// Copy the stuff anyway
 	} else if dstListErr != nil {
 		fs.Errorf(job.dstRemote, "error reading destination directory: %v", dstListErr)
 		fs.CountError(dstListErr)
-		return nil
+		return nil, dstListErr
+	}
+
+	// If NoTraverse is set, then try to find a matching object
+	// for each item in the srcList
+	if m.NoTraverse {
+		for _, src := range srcList {
+			if srcObj, ok := src.(fs.Object); ok {
+				leaf := path.Base(srcObj.Remote())
+				dstObj, err := m.Fdst.NewObject(m.Ctx, path.Join(job.dstRemote, leaf))
+				if err == nil {
+					dstList = append(dstList, dstObj)
+				}
+			}
+		}
 	}
 
 	// Work out what to do and do it
 	srcOnly, dstOnly, matches := matchListings(srcList, dstList, m.transforms)
 	for _, src := range srcOnly {
 		if m.aborting() {
-			return nil
+			return nil, m.Ctx.Err()
 		}
-		recurse := m.callback.SrcOnly(src)
+		recurse := m.Callback.SrcOnly(src)
 		if recurse && job.srcDepth > 0 {
 			jobs = append(jobs, listDirJob{
 				srcRemote: src.Remote(),
+				dstRemote: src.Remote(),
 				srcDepth:  job.srcDepth - 1,
 				noDst:     true,
 			})
@@ -389,11 +437,12 @@ func (m *March) processJob(job listDirJob) (jobs []listDirJob) {
 	}
 	for _, dst := range dstOnly {
 		if m.aborting() {
-			return nil
+			return nil, m.Ctx.Err()
 		}
-		recurse := m.callback.DstOnly(dst)
+		recurse := m.Callback.DstOnly(dst)
 		if recurse && job.dstDepth > 0 {
 			jobs = append(jobs, listDirJob{
+				srcRemote: dst.Remote(),
 				dstRemote: dst.Remote(),
 				dstDepth:  job.dstDepth - 1,
 				noSrc:     true,
@@ -402,9 +451,9 @@ func (m *March) processJob(job listDirJob) (jobs []listDirJob) {
 	}
 	for _, match := range matches {
 		if m.aborting() {
-			return nil
+			return nil, m.Ctx.Err()
 		}
-		recurse := m.callback.Match(match.dst, match.src)
+		recurse := m.Callback.Match(m.Ctx, match.dst, match.src)
 		if recurse && job.srcDepth > 0 && job.dstDepth > 0 {
 			jobs = append(jobs, listDirJob{
 				srcRemote: match.src.Remote(),
@@ -414,5 +463,5 @@ func (m *March) processJob(job listDirJob) (jobs []listDirJob) {
 			})
 		}
 	}
-	return jobs
+	return jobs, nil
 }
