@@ -5,6 +5,7 @@ package fstest
 
 import (
 	"bytes"
+	"context"
 	"flag"
 	"fmt"
 	"io"
@@ -21,11 +22,12 @@ import (
 	"testing"
 	"time"
 
-	"github.com/ncw/rclone/fs"
-	"github.com/ncw/rclone/fs/accounting"
-	"github.com/ncw/rclone/fs/config"
-	"github.com/ncw/rclone/fs/hash"
-	"github.com/ncw/rclone/fs/walk"
+	"github.com/rclone/rclone/fs"
+	"github.com/rclone/rclone/fs/accounting"
+	"github.com/rclone/rclone/fs/config"
+	"github.com/rclone/rclone/fs/hash"
+	"github.com/rclone/rclone/fs/walk"
+	"github.com/rclone/rclone/lib/random"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
 	"golang.org/x/text/unicode/norm"
@@ -34,13 +36,14 @@ import (
 // Globals
 var (
 	RemoteName      = flag.String("remote", "", "Remote to test with, defaults to local filesystem")
-	SubDir          = flag.Bool("subdir", false, "Set to test with a sub directory")
 	Verbose         = flag.Bool("verbose", false, "Set to enable logging")
 	DumpHeaders     = flag.Bool("dump-headers", false, "Set to dump headers (needs -verbose)")
 	DumpBodies      = flag.Bool("dump-bodies", false, "Set to dump bodies (needs -verbose)")
 	Individual      = flag.Bool("individual", false, "Make individual bucket/container/directory for each test - much slower")
 	LowLevelRetries = flag.Int("low-level-retries", 10, "Number of low level retries")
 	UseListR        = flag.Bool("fast-list", false, "Use recursive list if available. Uses more memory but fewer transactions.")
+	// SizeLimit signals tests to skip maximum test file size and skip inappropriate runs
+	SizeLimit = flag.Int64("size-limit", 0, "Limit maximum test file size")
 	// ListRetries is the number of times to retry a listing to overcome eventual consistency
 	ListRetries = flag.Int("list-retries", 6, "Number or times to retry listing")
 	// MatchTestRemote matches the remote names used for testing
@@ -85,7 +88,6 @@ type Item struct {
 	Hashes  map[hash.Type]string
 	ModTime time.Time
 	Size    int64
-	WinPath string
 }
 
 // NewItem creates an item from a string content
@@ -127,7 +129,7 @@ func (i *Item) CheckHashes(t *testing.T, obj fs.Object) {
 	types := obj.Fs().Hashes().Array()
 	for _, Hash := range types {
 		// Check attributes
-		sum, err := obj.Hash(Hash)
+		sum, err := obj.Hash(context.Background(), Hash)
 		require.NoError(t, err)
 		assert.True(t, hash.Equals(i.Hashes[Hash], sum), fmt.Sprintf("%s/%s: %v hash incorrect - expecting %q got %q", obj.Fs().String(), obj.Remote(), Hash, i.Hashes[Hash], sum))
 	}
@@ -137,18 +139,7 @@ func (i *Item) CheckHashes(t *testing.T, obj fs.Object) {
 func (i *Item) Check(t *testing.T, obj fs.Object, precision time.Duration) {
 	i.CheckHashes(t, obj)
 	assert.Equal(t, i.Size, obj.Size(), fmt.Sprintf("%s: size incorrect file=%d vs obj=%d", i.Path, i.Size, obj.Size()))
-	i.CheckModTime(t, obj, obj.ModTime(), precision)
-}
-
-// WinPath converts a path into a windows safe path
-func WinPath(s string) string {
-	return strings.Map(func(r rune) rune {
-		switch r {
-		case '<', '>', '"', '|', '?', '*', ':':
-			return '_'
-		}
-		return r
-	}, s)
+	i.CheckModTime(t, obj, obj.ModTime(context.Background()), precision)
 }
 
 // Normalize runs a utf8 normalization on the string if running on OS
@@ -178,7 +169,6 @@ func NewItems(items []Item) *Items {
 	// Fill up byName
 	for i := range items {
 		is.byName[Normalize(items[i].Path)] = &items[i]
-		is.byNameAlt[Normalize(items[i].WinPath)] = &items[i]
 	}
 	return is
 }
@@ -193,7 +183,6 @@ func (is *Items) Find(t *testing.T, obj fs.Object, precision time.Duration) {
 	}
 	if i != nil {
 		delete(is.byName, i.Path)
-		delete(is.byName, i.WinPath)
 		i.Check(t, obj, precision)
 	}
 }
@@ -211,21 +200,14 @@ func (is *Items) Done(t *testing.T) {
 // makeListingFromItems returns a string representation of the items
 //
 // it returns two possible strings, one normal and one for windows
-func makeListingFromItems(items []Item) (string, string) {
-	nameLengths1 := make([]string, len(items))
-	nameLengths2 := make([]string, len(items))
+func makeListingFromItems(items []Item) string {
+	nameLengths := make([]string, len(items))
 	for i, item := range items {
-		remote1 := Normalize(item.Path)
-		remote2 := remote1
-		if item.WinPath != "" {
-			remote2 = item.WinPath
-		}
-		nameLengths1[i] = fmt.Sprintf("%s (%d)", remote1, item.Size)
-		nameLengths2[i] = fmt.Sprintf("%s (%d)", remote2, item.Size)
+		remote := Normalize(item.Path)
+		nameLengths[i] = fmt.Sprintf("%s (%d)", remote, item.Size)
 	}
-	sort.Strings(nameLengths1)
-	sort.Strings(nameLengths2)
-	return strings.Join(nameLengths1, ", "), strings.Join(nameLengths2, ", ")
+	sort.Strings(nameLengths)
+	return strings.Join(nameLengths, ", ")
 }
 
 // makeListingFromObjects returns a string representation of the objects
@@ -262,34 +244,37 @@ func filterEmptyDirs(t *testing.T, items []Item, expectedDirs []string) (newExpe
 	return newExpectedDirs
 }
 
-// CheckListingWithPrecision checks the fs to see if it has the
+// CheckListingWithRoot checks the fs to see if it has the
 // expected contents with the given precision.
 //
 // If expectedDirs is non nil then we check those too.  Note that no
 // directories returned is also OK as some remotes don't return
 // directories.
-func CheckListingWithPrecision(t *testing.T, f fs.Fs, items []Item, expectedDirs []string, precision time.Duration) {
+//
+// dir is the directory used for the listing.
+func CheckListingWithRoot(t *testing.T, f fs.Fs, dir string, items []Item, expectedDirs []string, precision time.Duration) {
 	if expectedDirs != nil && !f.Features().CanHaveEmptyDirectories {
 		expectedDirs = filterEmptyDirs(t, items, expectedDirs)
 	}
 	is := NewItems(items)
-	oldErrors := accounting.Stats.GetErrors()
+	ctx := context.Background()
+	oldErrors := accounting.Stats(ctx).GetErrors()
 	var objs []fs.Object
 	var dirs []fs.Directory
 	var err error
 	var retries = *ListRetries
 	sleep := time.Second / 2
-	wantListing1, wantListing2 := makeListingFromItems(items)
+	wantListing := makeListingFromItems(items)
 	gotListing := "<unset>"
 	listingOK := false
 	for i := 1; i <= retries; i++ {
-		objs, dirs, err = walk.GetAll(f, "", true, -1)
+		objs, dirs, err = walk.GetAll(ctx, f, dir, true, -1)
 		if err != nil && err != fs.ErrorDirNotFound {
 			t.Fatalf("Error listing: %v", err)
 		}
 
 		gotListing = makeListingFromObjects(objs)
-		listingOK = wantListing1 == gotListing || wantListing2 == gotListing
+		listingOK = wantListing == gotListing
 		if listingOK && (expectedDirs == nil || len(dirs) == len(expectedDirs)) {
 			// Put an extra sleep in if we did any retries just to make sure it really
 			// is consistent (here is looking at you Amazon Drive!)
@@ -308,25 +293,25 @@ func CheckListingWithPrecision(t *testing.T, f fs.Fs, items []Item, expectedDirs
 			doDirCacheFlush()
 		}
 	}
-	assert.True(t, listingOK, fmt.Sprintf("listing wrong, want\n  %s or\n  %s got\n  %s", wantListing1, wantListing2, gotListing))
+	assert.True(t, listingOK, fmt.Sprintf("listing wrong, want\n  %s got\n  %s", wantListing, gotListing))
 	for _, obj := range objs {
 		require.NotNil(t, obj)
 		is.Find(t, obj, precision)
 	}
 	is.Done(t)
 	// Don't notice an error when listing an empty directory
-	if len(items) == 0 && oldErrors == 0 && accounting.Stats.GetErrors() == 1 {
-		accounting.Stats.ResetErrors()
+	if len(items) == 0 && oldErrors == 0 && accounting.Stats(ctx).GetErrors() == 1 {
+		accounting.Stats(ctx).ResetErrors()
 	}
 	// Check the directories
 	if expectedDirs != nil {
 		expectedDirsCopy := make([]string, len(expectedDirs))
 		for i, dir := range expectedDirs {
-			expectedDirsCopy[i] = WinPath(Normalize(dir))
+			expectedDirsCopy[i] = Normalize(dir)
 		}
 		actualDirs := []string{}
 		for _, dir := range dirs {
-			actualDirs = append(actualDirs, WinPath(Normalize(dir.Remote())))
+			actualDirs = append(actualDirs, Normalize(dir.Remote()))
 		}
 		sort.Strings(actualDirs)
 		sort.Strings(expectedDirsCopy)
@@ -334,6 +319,16 @@ func CheckListingWithPrecision(t *testing.T, f fs.Fs, items []Item, expectedDirs
 	}
 }
 
+// CheckListingWithPrecision checks the fs to see if it has the
+// expected contents with the given precision.
+//
+// If expectedDirs is non nil then we check those too.  Note that no
+// directories returned is also OK as some remotes don't return
+// directories.
+func CheckListingWithPrecision(t *testing.T, f fs.Fs, items []Item, expectedDirs []string, precision time.Duration) {
+	CheckListingWithRoot(t, f, "", items, expectedDirs, precision)
+}
+
 // CheckListing checks the fs to see if it has the expected contents
 func CheckListing(t *testing.T, f fs.Fs, items []Item) {
 	precision := f.Precision()
@@ -346,6 +341,49 @@ func CheckItems(t *testing.T, f fs.Fs, items ...Item) {
 	CheckListingWithPrecision(t, f, items, nil, fs.GetModifyWindow(f))
 }
 
+// CompareItems compares a set of DirEntries to a slice of items and a list of dirs
+// The modtimes are compared with the precision supplied
+func CompareItems(t *testing.T, entries fs.DirEntries, items []Item, expectedDirs []string, precision time.Duration, what string) {
+	is := NewItems(items)
+	var objs []fs.Object
+	var dirs []fs.Directory
+	wantListing := makeListingFromItems(items)
+	for _, entry := range entries {
+		switch x := entry.(type) {
+		case fs.Directory:
+			dirs = append(dirs, x)
+		case fs.Object:
+			objs = append(objs, x)
+			// do nothing
+		default:
+			t.Fatalf("unknown object type %T", entry)
+		}
+	}
+
+	gotListing := makeListingFromObjects(objs)
+	listingOK := wantListing == gotListing
+	assert.True(t, listingOK, fmt.Sprintf("%s not equal, want\n  %s got\n  %s", what, wantListing, gotListing))
+	for _, obj := range objs {
+		require.NotNil(t, obj)
+		is.Find(t, obj, precision)
+	}
+	is.Done(t)
+	// Check the directories
+	if expectedDirs != nil {
+		expectedDirsCopy := make([]string, len(expectedDirs))
+		for i, dir := range expectedDirs {
+			expectedDirsCopy[i] = Normalize(dir)
+		}
+		actualDirs := []string{}
+		for _, dir := range dirs {
+			actualDirs = append(actualDirs, Normalize(dir.Remote()))
+		}
+		sort.Strings(actualDirs)
+		sort.Strings(expectedDirsCopy)
+		assert.Equal(t, expectedDirsCopy, actualDirs, "directories not equal")
+	}
+}
+
 // Time parses a time string or logs a fatal error
 func Time(timeString string) time.Time {
 	t, err := time.Parse(time.RFC3339Nano, timeString)
@@ -355,24 +393,6 @@ func Time(timeString string) time.Time {
 	return t
 }
 
-// RandomString create a random string for test purposes
-func RandomString(n int) string {
-	const (
-		vowel     = "aeiou"
-		consonant = "bcdfghjklmnpqrstvwxyz"
-		digit     = "0123456789"
-	)
-	pattern := []string{consonant, vowel, consonant, vowel, consonant, vowel, consonant, digit}
-	out := make([]byte, n)
-	p := 0
-	for i := range out {
-		source := pattern[p]
-		p = (p + 1) % len(pattern)
-		out[i] = source[rand.Intn(len(source))]
-	}
-	return string(out)
-}
-
 // LocalRemote creates a temporary directory name for local remotes
 func LocalRemote() (path string, err error) {
 	path, err = ioutil.TempDir("", "rclone")
@@ -401,7 +421,7 @@ func RandomRemoteName(remoteName string) (string, string, error) {
 		if !strings.HasSuffix(remoteName, ":") {
 			remoteName += "/"
 		}
-		leafName = "rclone-test-" + RandomString(24)
+		leafName = "rclone-test-" + random.String(24)
 		if !MatchTestRemote.MatchString(leafName) {
 			log.Fatalf("%q didn't match the test remote name regexp", leafName)
 		}
@@ -411,28 +431,22 @@ func RandomRemoteName(remoteName string) (string, string, error) {
 }
 
 // RandomRemote makes a random bucket or subdirectory on the remote
+// from the -remote parameter
 //
 // Call the finalise function returned to Purge the fs at the end (and
 // the parent if necessary)
 //
 // Returns the remote, its url, a finaliser and an error
-func RandomRemote(remoteName string, subdir bool) (fs.Fs, string, func(), error) {
+func RandomRemote() (fs.Fs, string, func(), error) {
 	var err error
 	var parentRemote fs.Fs
+	remoteName := *RemoteName
 
 	remoteName, _, err = RandomRemoteName(remoteName)
 	if err != nil {
 		return nil, "", nil, err
 	}
 
-	if subdir {
-		parentRemote, err = fs.NewFs(remoteName)
-		if err != nil {
-			return nil, "", nil, err
-		}
-		remoteName += "/rclone-test-subdir-" + RandomString(8)
-	}
-
 	remote, err := fs.NewFs(remoteName)
 	if err != nil {
 		return nil, "", nil, err
@@ -456,26 +470,24 @@ func RandomRemote(remoteName string, subdir bool) (fs.Fs, string, func(), error)
 //
 // It logs errors rather than returning them
 func Purge(f fs.Fs) {
+	ctx := context.Background()
 	var err error
 	doFallbackPurge := true
 	if doPurge := f.Features().Purge; doPurge != nil {
 		doFallbackPurge = false
 		fs.Debugf(f, "Purge remote")
-		err = doPurge()
+		err = doPurge(ctx)
 		if err == fs.ErrorCantPurge {
 			doFallbackPurge = true
 		}
 	}
 	if doFallbackPurge {
 		dirs := []string{""}
-		err = walk.Walk(f, "", true, -1, func(dirPath string, entries fs.DirEntries, err error) error {
-			if err != nil {
-				log.Printf("purge walk returned error: %v", err)
-				return nil
-			}
+		err = walk.ListR(ctx, f, "", true, -1, walk.ListAll, func(entries fs.DirEntries) error {
+			var err error
 			entries.ForObject(func(obj fs.Object) {
 				fs.Debugf(f, "Purge object %q", obj.Remote())
-				err = obj.Remove()
+				err = obj.Remove(ctx)
 				if err != nil {
 					log.Printf("purge failed to remove %q: %v", obj.Remote(), err)
 				}
@@ -489,7 +501,7 @@ func Purge(f fs.Fs) {
 		for i := len(dirs) - 1; i >= 0; i-- {
 			dir := dirs[i]
 			fs.Debugf(f, "Purge dir %q", dir)
-			err := f.Rmdir(dir)
+			err := f.Rmdir(ctx, dir)
 			if err != nil {
 				log.Printf("purge failed to rmdir %q: %v", dir, err)
 			}
