@@ -23,16 +23,10 @@ import (
 	"strings"
 	"time"
 
-	_ "github.com/ncw/rclone/backend/all" // import all fs
-	"github.com/ncw/rclone/lib/pacer"
+	_ "github.com/rclone/rclone/backend/all" // import all fs
+	"github.com/rclone/rclone/lib/pacer"
 )
 
-type remoteConfig struct {
-	Name     string
-	SubDir   bool
-	FastList bool
-}
-
 var (
 	// Flags
 	maxTries     = flag.Int("maxtries", 5, "Number of times to try each test")
@@ -151,6 +145,7 @@ func main() {
 	// Log and exit
 	report.End()
 	report.LogSummary()
+	report.LogJSON()
 	report.LogHTML()
 	report.EmailHTML()
 	report.Upload()
