@@ -1,6 +1,7 @@
 package vfs
 
 import (
+	"context"
 	"os"
 	"path"
 	"sort"
@@ -8,10 +9,13 @@ import (
 	"sync"
 	"time"
 
-	"github.com/ncw/rclone/fs"
-	"github.com/ncw/rclone/fs/list"
-	"github.com/ncw/rclone/fs/walk"
 	"github.com/pkg/errors"
+	"github.com/rclone/rclone/fs"
+	"github.com/rclone/rclone/fs/dirtree"
+	"github.com/rclone/rclone/fs/list"
+	"github.com/rclone/rclone/fs/log"
+	"github.com/rclone/rclone/fs/operations"
+	"github.com/rclone/rclone/fs/walk"
 )
 
 // Dir represents a directory entry
@@ -35,7 +39,7 @@ func newDir(vfs *VFS, f fs.Fs, parent *Dir, fsDir fs.Directory) *Dir {
 		parent:  parent,
 		entry:   fsDir,
 		path:    fsDir.Remote(),
-		modTime: fsDir.ModTime(),
+		modTime: fsDir.ModTime(context.TODO()),
 		inode:   newInode(),
 		items:   make(map[string]Node),
 	}
@@ -93,10 +97,53 @@ func (d *Dir) Node() Node {
 	return d
 }
 
+// forgetDirPath clears the cache for itself and all subdirectories if
+// they match the given path. The path is specified relative from the
+// directory it is called from.
+//
+// It does not invalidate or clear the cache of the parent directory.
+func (d *Dir) forgetDirPath(relativePath string) {
+	if dir := d.cachedDir(relativePath); dir != nil {
+		dir.walk(func(dir *Dir) {
+			fs.Debugf(dir.path, "forgetting directory cache")
+			dir.read = time.Time{}
+			dir.items = make(map[string]Node)
+		})
+	}
+}
+
 // ForgetAll ensures the directory and all its children are purged
 // from the cache.
+//
+// It does not invalidate or clear the cache of the parent directory.
 func (d *Dir) ForgetAll() {
-	d.ForgetPath("", fs.EntryDirectory)
+	d.forgetDirPath("")
+}
+
+// invalidateDir invalidates the directory cache for absPath relative to this dir
+func (d *Dir) invalidateDir(absPath string) {
+	node := d.vfs.root.cachedNode(absPath)
+	if dir, ok := node.(*Dir); ok {
+		dir.mu.Lock()
+		if !dir.read.IsZero() {
+			fs.Debugf(dir.path, "invalidating directory cache")
+			dir.read = time.Time{}
+		}
+		dir.mu.Unlock()
+	}
+}
+
+// changeNotify invalidates the directory cache for the relativePath
+// passed in.
+//
+// if entryType is a directory it invalidates the parent of the directory too.
+func (d *Dir) changeNotify(relativePath string, entryType fs.EntryType) {
+	defer log.Trace(d.path, "relativePath=%q, type=%v", relativePath, entryType)("")
+	absPath := path.Join(d.path, relativePath)
+	d.invalidateDir(findParent(absPath))
+	if entryType == fs.EntryDirectory {
+		d.invalidateDir(absPath)
+	}
 }
 
 // ForgetPath clears the cache for itself and all subdirectories if
@@ -106,30 +153,12 @@ func (d *Dir) ForgetAll() {
 // It is not possible to traverse the directory tree upwards, i.e.
 // you cannot clear the cache for the Dir's ancestors or siblings.
 func (d *Dir) ForgetPath(relativePath string, entryType fs.EntryType) {
+	defer log.Trace(d.path, "relativePath=%q, type=%v", relativePath, entryType)("")
 	if absPath := path.Join(d.path, relativePath); absPath != "" {
-		parent := path.Dir(absPath)
-		if parent == "." || parent == "/" {
-			parent = ""
-		}
-		parentNode := d.vfs.root.cachedNode(parent)
-		if dir, ok := parentNode.(*Dir); ok {
-			dir.mu.Lock()
-			if !dir.read.IsZero() {
-				fs.Debugf(dir.path, "invalidating directory cache")
-				dir.read = time.Time{}
-			}
-			dir.mu.Unlock()
-		}
+		d.invalidateDir(findParent(absPath))
 	}
-
 	if entryType == fs.EntryDirectory {
-		if dir := d.cachedDir(relativePath); dir != nil {
-			dir.walk(func(dir *Dir) {
-				fs.Debugf(dir.path, "forgetting directory cache")
-				dir.read = time.Time{}
-				dir.items = make(map[string]Node)
-			})
-		}
+		d.forgetDirPath(relativePath)
 	}
 }
 
@@ -176,7 +205,7 @@ func (d *Dir) rename(newParent *Dir, fsDir fs.Directory) {
 	d.parent = newParent
 	d.entry = fsDir
 	d.path = fsDir.Remote()
-	d.modTime = fsDir.ModTime()
+	d.modTime = fsDir.ModTime(context.TODO())
 	d.read = time.Time{}
 }
 
@@ -206,7 +235,7 @@ func (d *Dir) _readDir() error {
 	} else {
 		return nil
 	}
-	entries, err := list.DirSorted(d.f, false, d.path)
+	entries, err := list.DirSorted(context.TODO(), d.f, false, d.path)
 	if err == fs.ErrorDirNotFound {
 		// We treat directory not found as empty because we
 		// create directories on the fly
@@ -225,13 +254,13 @@ func (d *Dir) _readDir() error {
 
 // update d.items for each dir in the DirTree below this one and
 // set the last read time - must be called with the lock held
-func (d *Dir) _readDirFromDirTree(dirTree walk.DirTree, when time.Time) error {
+func (d *Dir) _readDirFromDirTree(dirTree dirtree.DirTree, when time.Time) error {
 	return d._readDirFromEntries(dirTree[d.path], dirTree, when)
 }
 
 // update d.items and if dirTree is not nil update each dir in the DirTree below this one and
 // set the last read time - must be called with the lock held
-func (d *Dir) _readDirFromEntries(entries fs.DirEntries, dirTree walk.DirTree, when time.Time) error {
+func (d *Dir) _readDirFromEntries(entries fs.DirEntries, dirTree dirtree.DirTree, when time.Time) error {
 	var err error
 	// Cache the items by name
 	found := make(map[string]struct{})
@@ -289,14 +318,17 @@ func (d *Dir) _readDirFromEntries(entries fs.DirEntries, dirTree walk.DirTree, w
 // readDirTree forces a refresh of the complete directory tree
 func (d *Dir) readDirTree() error {
 	d.mu.Lock()
-	defer d.mu.Unlock()
+	f, path := d.f, d.path
+	d.mu.Unlock()
 	when := time.Now()
-	d.read = time.Time{}
-	fs.Debugf(d.path, "Reading directory tree")
-	dt, err := walk.NewDirTree(d.f, d.path, false, -1)
+	fs.Debugf(path, "Reading directory tree")
+	dt, err := walk.NewDirTree(context.TODO(), f, path, false, -1)
 	if err != nil {
 		return err
 	}
+	d.mu.Lock()
+	defer d.mu.Unlock()
+	d.read = time.Time{}
 	err = d._readDirFromDirTree(dt, when)
 	if err != nil {
 		return err
@@ -317,6 +349,8 @@ func (d *Dir) readDir() error {
 // stat a single item in the directory
 //
 // returns ENOENT if not found.
+// returns a custom error if directory on a case-insensitive file system
+// contains files with names that differ only by case.
 func (d *Dir) stat(leaf string) (Node, error) {
 	d.mu.Lock()
 	defer d.mu.Unlock()
@@ -325,6 +359,22 @@ func (d *Dir) stat(leaf string) (Node, error) {
 		return nil, err
 	}
 	item, ok := d.items[leaf]
+
+	if !ok && d.vfs.Opt.CaseInsensitive {
+		leafLower := strings.ToLower(leaf)
+		for name, node := range d.items {
+			if strings.ToLower(name) == leafLower {
+				if ok {
+					// duplicate case insensitive match is an error
+					return nil, errors.Errorf("duplicate filename %q detected with --vfs-case-insensitive set", leaf)
+				}
+				// found a case insenstive match
+				ok = true
+				item = node
+			}
+		}
+	}
+
 	if !ok {
 		return nil, ENOENT
 	}
@@ -457,8 +507,23 @@ func (d *Dir) Mkdir(name string) (*Dir, error) {
 		return nil, EROFS
 	}
 	path := path.Join(d.path, name)
+	node, err := d.stat(name)
+	switch err {
+	case ENOENT:
+		// not found, carry on
+	case nil:
+		// found so check what it is
+		if node.IsDir() {
+			return node.(*Dir), err
+		}
+		return nil, EEXIST
+	default:
+		// a different error - report
+		fs.Errorf(d, "Dir.Mkdir failed to read directory: %v", err)
+		return nil, err
+	}
 	// fs.Debugf(path, "Dir.Mkdir")
-	err := d.f.Mkdir(path)
+	err = d.f.Mkdir(context.TODO(), path)
 	if err != nil {
 		fs.Errorf(d, "Dir.Mkdir failed to create directory: %v", err)
 		return nil, err
@@ -486,7 +551,7 @@ func (d *Dir) Remove() error {
 		return ENOTEMPTY
 	}
 	// remove directory
-	err = d.f.Rmdir(d.path)
+	err = d.f.Rmdir(context.TODO(), d.path)
 	if err != nil {
 		fs.Errorf(d, "Dir.Remove failed to remove directory: %v", err)
 		return err
@@ -556,7 +621,7 @@ func (d *Dir) Rename(oldName, newName string, destDir *Dir) error {
 	switch x := oldNode.DirEntry().(type) {
 	case nil:
 		if oldFile, ok := oldNode.(*File); ok {
-			if err = oldFile.rename(destDir, newName); err != nil {
+			if err = oldFile.rename(context.TODO(), destDir, newName); err != nil {
 				fs.Errorf(oldPath, "Dir.Rename error: %v", err)
 				return err
 			}
@@ -566,7 +631,7 @@ func (d *Dir) Rename(oldName, newName string, destDir *Dir) error {
 		}
 	case fs.Object:
 		if oldFile, ok := oldNode.(*File); ok {
-			if err = oldFile.rename(destDir, newName); err != nil {
+			if err = oldFile.rename(context.TODO(), destDir, newName); err != nil {
 				fs.Errorf(oldPath, "Dir.Rename error: %v", err)
 				return err
 			}
@@ -576,20 +641,20 @@ func (d *Dir) Rename(oldName, newName string, destDir *Dir) error {
 			return err
 		}
 	case fs.Directory:
-		doDirMove := d.f.Features().DirMove
-		if doDirMove == nil {
-			err := errors.Errorf("Fs %q can't rename directories (no DirMove)", d.f)
+		features := d.f.Features()
+		if features.DirMove == nil && features.Move == nil && features.Copy == nil {
+			err := errors.Errorf("Fs %q can't rename directories (no DirMove, Move or Copy)", d.f)
 			fs.Errorf(oldPath, "Dir.Rename error: %v", err)
 			return err
 		}
 		srcRemote := x.Remote()
 		dstRemote := newPath
-		err = doDirMove(d.f, srcRemote, dstRemote)
+		err = operations.DirMove(context.TODO(), d.f, srcRemote, dstRemote)
 		if err != nil {
 			fs.Errorf(oldPath, "Dir.Rename error: %v", err)
 			return err
 		}
-		newDir := fs.NewDirCopy(x).SetRemote(newPath)
+		newDir := fs.NewDirCopy(context.TODO(), x).SetRemote(newPath)
 		// Update the node with the new details
 		if oldNode != nil {
 			if oldDir, ok := oldNode.(*Dir); ok {
@@ -599,7 +664,7 @@ func (d *Dir) Rename(oldName, newName string, destDir *Dir) error {
 		}
 	default:
 		err = errors.Errorf("unknown type %T", oldNode)
-		fs.Errorf(d.path, "Dir.ReadDirAll error: %v", err)
+		fs.Errorf(d.path, "Dir.Rename error: %v", err)
 		return err
 	}
 
