@@ -1,16 +1,17 @@
 package vfs
 
 import (
+	"context"
 	"os"
 	"path"
 	"sync"
 	"sync/atomic"
 	"time"
 
-	"github.com/ncw/rclone/fs"
-	"github.com/ncw/rclone/fs/log"
-	"github.com/ncw/rclone/fs/operations"
 	"github.com/pkg/errors"
+	"github.com/rclone/rclone/fs"
+	"github.com/rclone/rclone/fs/log"
+	"github.com/rclone/rclone/fs/operations"
 )
 
 // File represents a file
@@ -19,17 +20,17 @@ type File struct {
 	size  int64  // size of file - read and written with atomic int64 - must be 64 bit aligned
 	d     *Dir   // parent directory - read only
 
-	mu                sync.Mutex   // protects the following
-	o                 fs.Object    // NB o may be nil if file is being written
-	leaf              string       // leaf name of the object
-	rwOpenCount       int          // number of open files on this handle
-	writers           []Handle     // writers for this file
-	nwriters          int32        // len(writers) which is read/updated with atomic
-	readWriters       int          // how many RWFileHandle are open for writing
-	readWriterClosing bool         // is a RWFileHandle currently cosing?
-	modified          bool         // has the cache file be modified by a RWFileHandle?
-	pendingModTime    time.Time    // will be applied once o becomes available, i.e. after file was written
-	pendingRenameFun  func() error // will be run/renamed after all writers close
+	mu                sync.Mutex                      // protects the following
+	o                 fs.Object                       // NB o may be nil if file is being written
+	leaf              string                          // leaf name of the object
+	rwOpenCount       int                             // number of open files on this handle
+	writers           []Handle                        // writers for this file
+	nwriters          int32                           // len(writers) which is read/updated with atomic
+	readWriters       int                             // how many RWFileHandle are open for writing
+	readWriterClosing bool                            // is a RWFileHandle currently cosing?
+	modified          bool                            // has the cache file be modified by a RWFileHandle?
+	pendingModTime    time.Time                       // will be applied once o becomes available, i.e. after file was written
+	pendingRenameFun  func(ctx context.Context) error // will be run/renamed after all writers close
 
 	muRW sync.Mutex // synchonize RWFileHandle.openPending(), RWFileHandle.close() and File.Remove
 }
@@ -100,7 +101,7 @@ func (f *File) applyPendingRename() {
 		return
 	}
 	fs.Debugf(f.o, "Running delayed rename now")
-	if err := fun(); err != nil {
+	if err := fun(context.TODO()); err != nil {
 		fs.Errorf(f.Path(), "delayed File.Rename error: %v", err)
 	}
 }
@@ -108,21 +109,27 @@ func (f *File) applyPendingRename() {
 // rename attempts to immediately rename a file if there are no open writers.
 // Otherwise it will queue the rename operation on the remote until no writers
 // remain.
-func (f *File) rename(destDir *Dir, newName string) error {
+func (f *File) rename(ctx context.Context, destDir *Dir, newName string) error {
 	if features := f.d.f.Features(); features.Move == nil && features.Copy == nil {
 		err := errors.Errorf("Fs %q can't rename files (no server side Move or Copy)", f.d.f)
 		fs.Errorf(f.Path(), "Dir.Rename error: %v", err)
 		return err
 	}
 
-	renameCall := func() error {
+	renameCall := func(ctx context.Context) error {
 		newPath := path.Join(destDir.path, newName)
-		dstOverwritten, _ := f.d.f.NewObject(newPath)
-		newObject, err := operations.Move(f.d.f, dstOverwritten, newPath, f.o)
+		dstOverwritten, _ := f.d.f.NewObject(ctx, newPath)
+		newObject, err := operations.Move(ctx, f.d.f, dstOverwritten, newPath, f.o)
 		if err != nil {
 			fs.Errorf(f.Path(), "File.Rename error: %v", err)
 			return err
 		}
+		// newObject can be nil here for example if --dry-run
+		if newObject == nil {
+			err = errors.New("rename failed: nil object returned")
+			fs.Errorf(f.Path(), "File.Rename %v", err)
+			return err
+		}
 		// Update the node with the new details
 		fs.Debugf(f.o, "Updating file with %v %p", newObject, f)
 		// f.rename(destDir, newObject)
@@ -145,7 +152,7 @@ func (f *File) rename(destDir *Dir, newName string) error {
 		return nil
 	}
 
-	return renameCall()
+	return renameCall(ctx)
 }
 
 // addWriter adds a write handle to the file
@@ -246,7 +253,7 @@ func (f *File) ModTime() (modTime time.Time) {
 				return f.pendingModTime
 			}
 		} else {
-			return f.o.ModTime()
+			return f.o.ModTime(context.TODO())
 		}
 	}
 
@@ -304,7 +311,7 @@ func (f *File) applyPendingModTime() error {
 		return errors.New("Cannot apply ModTime, file object is not available")
 	}
 
-	err := f.o.SetModTime(f.pendingModTime)
+	err := f.o.SetModTime(context.TODO(), f.pendingModTime)
 	switch err {
 	case nil:
 		fs.Debugf(f.o, "File.applyPendingModTime OK")
@@ -331,9 +338,10 @@ func (f *File) setSize(n int64) {
 // Update the object when written and add it to the directory
 func (f *File) setObject(o fs.Object) {
 	f.mu.Lock()
-	defer f.mu.Unlock()
 	f.o = o
 	_ = f.applyPendingModTime()
+	f.mu.Unlock()
+
 	f.d.addObject(f)
 }
 
@@ -440,20 +448,23 @@ func (f *File) Sync() error {
 
 // Remove the file
 func (f *File) Remove() error {
-	f.mu.Lock()
-	defer f.mu.Unlock()
-	f.muRW.Lock()
-	defer f.muRW.Unlock()
 	if f.d.vfs.Opt.ReadOnly {
 		return EROFS
 	}
+	f.muRW.Lock() // muRW must be locked before mu to avoid
+	f.mu.Lock()   // deadlock in RWFileHandle.openPending and .close
 	if f.o != nil {
-		err := f.o.Remove()
+		err := f.o.Remove(context.TODO())
 		if err != nil {
 			fs.Errorf(f, "File.Remove file error: %v", err)
+			f.mu.Unlock()
+			f.muRW.Unlock()
 			return err
 		}
 	}
+	f.mu.Unlock()
+	f.muRW.Unlock()
+
 	// Remove the item from the directory listing
 	f.d.delObject(f.Name())
 	// Remove the object from the cache
@@ -535,11 +546,9 @@ func (f *File) Open(flags int) (fd Handle, err error) {
 		write = true
 	}
 
-	// FIXME discover if file is in cache or not?
-
 	// Open the correct sort of handle
 	CacheMode := f.d.vfs.Opt.CacheMode
-	if CacheMode >= CacheModeMinimal && f.d.vfs.cache.opens(f.Path()) > 0 {
+	if CacheMode >= CacheModeMinimal && (f.d.vfs.cache.opens(f.Path()) > 0 || f.d.vfs.cache.exists(f.Path())) {
 		fd, err = f.openRW(flags)
 	} else if read && write {
 		if CacheMode >= CacheModeMinimal {
@@ -591,6 +600,12 @@ func (f *File) Truncate(size int64) (err error) {
 		}
 		return err
 	}
+
+	// If no writers, and size is already correct then all done
+	if f.o.Size() == size {
+		return nil
+	}
+
 	fs.Debugf(f.o, "Truncating file")
 
 	// Otherwise if no writers then truncate the file by opening
