@@ -15,6 +15,8 @@ import (
 	"log"
 	"os"
 	"strings"
+
+	"github.com/rclone/rclone/lib/file"
 )
 
 // Interprets err into a vfs error
@@ -45,7 +47,7 @@ func test(fileName string, flags int, mode string) {
 	if !os.IsNotExist(err) {
 		log.Fatalf("File must not exist")
 	}
-	f, openNonExistentErr := os.OpenFile(fileName, flags, 0666)
+	f, openNonExistentErr := file.OpenFile(fileName, flags, 0666)
 
 	var readNonExistentErr error
 	var writeNonExistentErr error
@@ -65,7 +67,7 @@ func test(fileName string, flags int, mode string) {
 	}
 
 	// write the file
-	f, err = os.Create(fileName)
+	f, err = file.Create(fileName)
 	if err != nil {
 		log.Fatalf("failed to create: %v", err)
 	}
@@ -81,7 +83,7 @@ func test(fileName string, flags int, mode string) {
 
 	// then open file and try with file existing
 
-	f, openExistingErr := os.OpenFile(fileName, flags, 0666)
+	f, openExistingErr := file.OpenFile(fileName, flags, 0666)
 	var readExistingErr error
 	var writeExistingErr error
 	if openExistingErr == nil {
@@ -100,7 +102,7 @@ func test(fileName string, flags int, mode string) {
 	}
 
 	// read the file
-	f, err = os.Open(fileName)
+	f, err = file.Open(fileName)
 	if err != nil {
 		log.Fatalf("failed to open: %v", err)
 	}
