@@ -1,6 +1,7 @@
 package vfs
 
 import (
+	"context"
 	"fmt"
 	"io"
 	"io/ioutil"
@@ -8,11 +9,11 @@ import (
 	"runtime"
 	"sync"
 
-	"github.com/ncw/rclone/fs"
-	"github.com/ncw/rclone/fs/accounting"
-	"github.com/ncw/rclone/fs/log"
-	"github.com/ncw/rclone/fs/operations"
 	"github.com/pkg/errors"
+	"github.com/rclone/rclone/fs"
+	"github.com/rclone/rclone/fs/log"
+	"github.com/rclone/rclone/fs/operations"
+	"github.com/rclone/rclone/lib/file"
 )
 
 // RWFileHandle is a handle that can be open for read and write.
@@ -84,10 +85,8 @@ func newRWFileHandle(d *Dir, f *File, remote string, flags int) (fh *RWFileHandl
 
 // copy an object to or from the remote while accounting for it
 func copyObj(f fs.Fs, dst fs.Object, remote string, src fs.Object) (newDst fs.Object, err error) {
-	if operations.NeedTransfer(dst, src) {
-		accounting.Stats.Transferring(src.Remote())
-		newDst, err = operations.Copy(f, dst, remote, src)
-		accounting.Stats.DoneTransferring(src.Remote(), err == nil)
+	if operations.NeedTransfer(context.TODO(), dst, src) {
+		newDst, err = operations.Copy(context.TODO(), f, dst, remote, src)
 	} else {
 		newDst = dst
 	}
@@ -114,7 +113,7 @@ func (fh *RWFileHandle) openPending(truncate bool) (err error) {
 		// If the remote object exists AND its cached file exists locally AND there are no
 		// other RW handles with it open, then attempt to update it.
 		if o != nil && fh.file.rwOpens() == 0 {
-			cacheObj, err := fh.d.vfs.cache.f.NewObject(fh.remote)
+			cacheObj, err := fh.d.vfs.cache.f.NewObject(context.TODO(), fh.remote)
 			if err == nil && cacheObj != nil {
 				_, err = copyObj(fh.d.vfs.cache.f, cacheObj, fh.remote, o)
 				if err != nil {
@@ -124,7 +123,7 @@ func (fh *RWFileHandle) openPending(truncate bool) (err error) {
 		}
 
 		// try to open a exising cache file
-		fd, err = os.OpenFile(fh.osPath, cacheFileOpenFlags&^os.O_CREATE, 0600)
+		fd, err = file.OpenFile(fh.osPath, cacheFileOpenFlags&^os.O_CREATE, 0600)
 		if os.IsNotExist(err) {
 			// cache file does not exist, so need to fetch it if we have an object to fetch
 			// it from
@@ -185,7 +184,7 @@ func (fh *RWFileHandle) openPending(truncate bool) (err error) {
 
 	if fd == nil {
 		fs.Debugf(fh.logPrefix(), "Opening cached copy with flags=%s", decodeOpenFlags(fh.flags))
-		fd, err = os.OpenFile(fh.osPath, cacheFileOpenFlags, 0600)
+		fd, err = file.OpenFile(fh.osPath, cacheFileOpenFlags, 0600)
 		if err != nil {
 			return errors.Wrap(err, "cache open file failed")
 		}
@@ -231,28 +230,14 @@ func (fh *RWFileHandle) modified() bool {
 	return true
 }
 
-// close the file handle returning EBADF if it has been
-// closed already.
+// flushWrites flushes any pending writes to cloud storage
 //
-// Must be called with fh.mu held
-//
-// Note that we leave the file around in the cache on error conditions
-// to give the user a chance to recover it.
-func (fh *RWFileHandle) close() (err error) {
-	defer log.Trace(fh.logPrefix(), "")("err=%v", &err)
-	fh.file.muRW.Lock()
-	defer fh.file.muRW.Unlock()
-
-	if fh.closed {
-		return ECLOSED
+// Must be called with fh.muRW held
+func (fh *RWFileHandle) flushWrites(closeFile bool) error {
+	if fh.closed && !closeFile {
+		return nil
 	}
-	fh.closed = true
-	defer func() {
-		if fh.opened {
-			fh.file.delRWOpen()
-		}
-		fh.d.vfs.cache.close(fh.remote)
-	}()
+
 	rdwrMode := fh.flags & accessModeMask
 	writer := rdwrMode != os.O_RDONLY
 
@@ -285,8 +270,8 @@ func (fh *RWFileHandle) close() (err error) {
 	}
 
 	// Close the underlying file
-	if fh.opened {
-		err = fh.File.Close()
+	if fh.opened && closeFile {
+		err := fh.File.Close()
 		if err != nil {
 			err = errors.Wrap(err, "failed to close cache file")
 			return err
@@ -295,7 +280,7 @@ func (fh *RWFileHandle) close() (err error) {
 
 	if isCopied {
 		// Transfer the temp file to the remote
-		cacheObj, err := fh.d.vfs.cache.f.NewObject(fh.remote)
+		cacheObj, err := fh.d.vfs.cache.f.NewObject(context.TODO(), fh.remote)
 		if err != nil {
 			err = errors.Wrap(err, "failed to find cache file")
 			fs.Errorf(fh.logPrefix(), "%v", err)
@@ -315,6 +300,32 @@ func (fh *RWFileHandle) close() (err error) {
 	return nil
 }
 
+// close the file handle returning EBADF if it has been
+// closed already.
+//
+// Must be called with fh.mu held
+//
+// Note that we leave the file around in the cache on error conditions
+// to give the user a chance to recover it.
+func (fh *RWFileHandle) close() (err error) {
+	defer log.Trace(fh.logPrefix(), "")("err=%v", &err)
+	fh.file.muRW.Lock()
+	defer fh.file.muRW.Unlock()
+
+	if fh.closed {
+		return ECLOSED
+	}
+	fh.closed = true
+	defer func() {
+		if fh.opened {
+			fh.file.delRWOpen()
+		}
+		fh.d.vfs.cache.close(fh.remote)
+	}()
+
+	return fh.flushWrites(true)
+}
+
 // Close closes the file
 func (fh *RWFileHandle) Close() error {
 	fh.mu.Lock()
@@ -347,7 +358,11 @@ func (fh *RWFileHandle) Flush() error {
 		fs.Debugf(fh.logPrefix(), "RWFileHandle.Flush ignoring flush on unwritten handle")
 		return nil
 	}
-	err := fh.close()
+
+	fh.file.muRW.Lock()
+	defer fh.file.muRW.Unlock()
+	err := fh.flushWrites(false)
+
 	if err != nil {
 		fs.Errorf(fh.logPrefix(), "RWFileHandle.Flush error: %v", err)
 	} else {
